@@ -87,12 +87,26 @@ void enqueue_flow_for_dumping(struct parser_base *p, struct flow_id* id, struct
 
 #define DUMPER_BUFFER_SIZE 100
 
+static void perform_dump(struct parser_base *p, struct flow_tuple *dump_tuple) {
+    uint8_t type = dump_tuple->flow_id->flow_type;
+    if (type == FLOW_TYPE_TCP) {
+        offload_tcp_flow(p, dump_tuple->flow_id, (struct tcp_flow *) dump_tuple->flow);
+        free_tcp_chunk(p, dump_tuple->flow_id, (struct tcp_flow *) dump_tuple->flow);
+    } else if (type == FLOW_TYPE_UDP) {
+        offload_udp_flow(p, dump_tuple->flow_id, (struct udp_flow *) dump_tuple->flow);
+        free_udp_chunk(p, dump_tuple->flow_id, (struct udp_flow *) dump_tuple->flow);
+    } else {
+        ERROR(p, "Bad flow id in dumper. Memory corrupted");
+    }
+
+    free(dump_tuple);
+}
+
 static void *dumper_thread(void *user) {
     struct parser_base *p = (struct parser_base *) user;
     int ret;
 
     struct flow_tuple *dump_tuple;
-    uint8_t type;
 
     struct flow_tuple * dumper_buffer[DUMPER_BUFFER_SIZE];
     size_t dumper_buffer_index = 0;
@@ -139,19 +153,7 @@ static void *dumper_thread(void *user) {
 
         for (size_t i = 0; i < dumper_buffer_index; i++) {
             dump_tuple = dumper_buffer[i];
-
-            type = dump_tuple->flow_id->flow_type;
-            if (type == FLOW_TYPE_TCP) {
-                offload_tcp_flow(p, dump_tuple->flow_id, (struct tcp_flow *) dump_tuple->flow);
-                free_tcp_chunk(p, dump_tuple->flow_id, (struct tcp_flow *) dump_tuple->flow);
-            } else if (type == FLOW_TYPE_UDP) {
-                offload_udp_flow(p, dump_tuple->flow_id, (struct udp_flow *) dump_tuple->flow);
-                free_udp_chunk(p, dump_tuple->flow_id, (struct udp_flow *) dump_tuple->flow);
-            } else {
-                ERROR(p, "Bad flow id in dumper. Memory corrupted");
-            }
-
-            free(dump_tuple);
+            perform_dump(p, dump_tuple);
         }
 
         dumper_buffer_index = 0;
@@ -160,6 +162,26 @@ static void *dumper_thread(void *user) {
     pthread_exit(NULL);
 }
 
+void force_synchronous_dump(struct parser_base *p) {
+    int ret;
+    if ((ret = pthread_mutex_lock(&p->dump_queue_mutex)) != 0) {
+        ERROR(p, "Unable to lock dump queue mutex: %s", strerror(ret));
+        return;
+    }
+
+    ERROR(p, "Forcing sync dump of %d flows", p->dump_queue->size);
+    while (p->dump_queue->size) {
+        struct flow_tuple *dump_tuple = kl_val(kl_begin(p->dump_queue));
+        perform_dump(p, dump_tuple);
+
+        kl_shift(32, p->dump_queue, NULL);
+    }
+
+    if ((ret = pthread_mutex_unlock(&p->dump_queue_mutex)) != 0) {
+        ERROR(p, "Unable to unlock dump queue mutex: %s", strerror(ret));
+    }
+}
+
 int init_dumper(struct parser_base *p) {
     pthread_attr_t attr;
     int ret;
